@@ -1,3 +1,5 @@
+# -*- coding: utf-8 -*-
+from __future__ import unicode_literals
 import os
 import pkg_resources
 import re
@@ -11,6 +13,7 @@ import threading
 import time
 import webbrowser
 from six.moves import shlex_quote as quote
+from unicodedata import normalize
 
 from dallinger import data
 from dallinger import db
@@ -128,8 +131,17 @@ class ExperimentFileSource(object):
             shutil.copy2(path, target_folder)
 
     def _walk(self):
+        # The GitClient and os.walk may return different representations of the
+        # same unicode characters, so we use unicodedata.normalize() for
+        # comparisons:
+        # list(name_from_git)
+        # ['å', ' ', 'f', 'i', 'l', 'e', '.', 't', 'x', 't']
+        # list(from_os_walk)
+        # ['a', '̊', ' ', 'f', 'i', 'l', 'e', '.', 't', 'x', 't']
         exclusions = exclusion_policy()
-        git_files = set([os.path.join(self.root, f) for f in self.git.files()])
+        git_files = {
+            os.path.join(self.root, normalize("NFC", f)) for f in self.git.files()
+        }
         for dirpath, dirnames, filenames in os.walk(self.root, topdown=True):
             current_exclusions = exclusions(dirpath, os.listdir(dirpath))
             # Modifying dirnames in-place will prune the subsequent files and
@@ -139,10 +151,13 @@ class ExperimentFileSource(object):
             legit_files = {
                 os.path.join(dirpath, f)
                 for f in filenames
-                if f not in current_exclusions and os.path.join(dirpath, f)
+                if f not in current_exclusions
             }
             if git_files:
-                legit_files = legit_files.intersection(git_files)
+                normalized = {
+                    normalize("NFC", six.text_type(f)): f for f in legit_files
+                }
+                legit_files = {v for k, v in normalized.items() if k in git_files}
             for legit in legit_files:
                 yield legit
 
@@ -302,6 +317,12 @@ def _handle_launch_data(url, error, delay=INITIAL_DELAY, attempts=MAX_ATTEMPTS):
         if launch_request.ok:
             return launch_data
 
+        error(
+            "Error accessing /launch ({}):\n{}".format(
+                launch_request.status_code, launch_request.text
+            )
+        )
+
         if remaining_attempt:
             delay = delay * BACKOFF_FACTOR
             next_attempt_count = attempts - (remaining_attempt - 1)
@@ -410,8 +431,9 @@ def deploy_sandbox_shared_setup(log, verbose=True, app=None, exp_config=None):
     git.push(remote="heroku", branch="HEAD:master")
 
     log("Scaling up the dynos...")
-    size = config.get("dyno_type")
+    default_size = config.get("dyno_type")
     for process in ["web", "worker"]:
+        size = config.get("dyno_type_" + process, default_size)
         qty = config.get("num_dynos_" + process)
         heroku_app.scale_up_dyno(process, qty, size)
     if config.get("clock_on"):
@@ -445,7 +467,7 @@ def _deploy_in_mode(mode, app, verbose, log):
     config.load()
 
     # Set the mode.
-    config.extend({"mode": mode, "logfile": u"-"})
+    config.extend({"mode": mode, "logfile": "-"})
 
     # Do shared setup.
     deploy_sandbox_shared_setup(log, verbose=verbose, app=app)
@@ -458,7 +480,7 @@ class HerokuLocalDeployment(object):
     dispatch = {}  # Subclass may provide handlers for Heroku process output
 
     def configure(self):
-        self.exp_config.update({"mode": u"debug", "loglevel": 0})
+        self.exp_config.update({"mode": "debug", "loglevel": 0})
 
     def setup(self):
         self.exp_id, self.tmp_dir = setup_experiment(
@@ -516,7 +538,7 @@ class DebugDeployment(HerokuLocalDeployment):
         r"{}".format(recruiters.CLOSE_RECRUITMENT_LOG_PREFIX): "recruitment_closed",
     }
 
-    def __init__(self, output, verbose, bot, proxy_port, exp_config):
+    def __init__(self, output, verbose, bot, proxy_port, exp_config, no_browsers=False):
         self.out = output
         self.verbose = verbose
         self.bot = bot
@@ -525,6 +547,7 @@ class DebugDeployment(HerokuLocalDeployment):
         self.original_dir = os.getcwd()
         self.complete = False
         self.status_thread = None
+        self.no_browsers = no_browsers
 
     def configure(self):
         super(DebugDeployment, self).configure()
@@ -538,7 +561,7 @@ class DebugDeployment(HerokuLocalDeployment):
         time.sleep(4)
         try:
             result = _handle_launch_data(
-                "{}/launch".format(base_url), error=self.out.error
+                "{}/launch".format(base_url), error=self.out.error, attempts=1
             )
         except Exception:
             # Show output from server
@@ -563,6 +586,9 @@ class DebugDeployment(HerokuLocalDeployment):
         person doing local debugging).
         """
         self.out.log("new recruitment request!")
+        if self.no_browsers:
+            self.out.log(recruiters.NEW_RECRUIT_LOG_PREFIX + ": " + match.group(1))
+            return
         url = match.group(1)
         if self.proxy_port is not None:
             self.out.log("Using proxy port {}".format(self.proxy_port))
@@ -574,6 +600,8 @@ class DebugDeployment(HerokuLocalDeployment):
 
         Start a thread to check the experiment summary.
         """
+        if self.no_browsers:
+            self.out.log(recruiters.CLOSE_RECRUITMENT_LOG_PREFIX)
         if self.status_thread is None:
             self.status_thread = threading.Thread(target=self.check_status)
             self.status_thread.start()
@@ -624,7 +652,7 @@ class LoaderDeployment(HerokuLocalDeployment):
         self.zip_path = None
 
     def configure(self):
-        self.exp_config.update({"mode": u"debug", "loglevel": 0})
+        self.exp_config.update({"mode": "debug", "loglevel": 0})
 
         self.zip_path = data.find_experiment_export(self.app_id)
         if self.zip_path is None:
