@@ -17,10 +17,13 @@ from dallinger.experiment_server.utils import success_response
 from dallinger.experiment_server.utils import crossdomain
 from dallinger.experiment_server.worker_events import worker_function
 from dallinger.heroku import tools as heroku_tools
-from dallinger.notifications import get_messenger
+from dallinger.notifications import get_mailer
+from dallinger.notifications import admin_notifier
 from dallinger.notifications import MessengerError
 from dallinger.models import Participant
 from dallinger.models import Recruitment
+from dallinger.mturk import MTurkQualificationRequirements
+from dallinger.mturk import MTurkQuestions
 from dallinger.mturk import MTurkService
 from dallinger.mturk import DuplicateQualificationNameError
 from dallinger.mturk import MTurkServiceException
@@ -82,6 +85,11 @@ class Recruiter(object):
         """Throw an error."""
         raise NotImplementedError
 
+    def compensate_worker(self, *args, **kwargs):
+        """A recruiter may provide a means to directly compensate a worker.
+        """
+        raise NotImplementedError
+
     def reward_bonus(self, assignment_id, amount, reason):
         """Throw an error."""
         raise NotImplementedError
@@ -449,7 +457,8 @@ class MTurkRecruiter(Recruiter):
             region_name=self.config.get("aws_region"),
             sandbox=self.config.get("mode") != "live",
         )
-        self.messenger = get_messenger(self.config)
+        self.notifies_admin = admin_notifier(self.config)
+        self.mailer = get_mailer(self.config)
         self._validate_config()
 
     def _validate_config(self):
@@ -504,26 +513,66 @@ class MTurkRecruiter(Recruiter):
             "reward": self.config.get("base_payment"),
             "duration_hours": self.config.get("duration"),
             "lifetime_days": self.config.get("lifetime"),
-            "ad_url": self.ad_url,
+            "question": MTurkQuestions.external(self.ad_url),
             "notification_url": self.notification_url,
-            "approve_requirement": self.config.get("approve_requirement"),
-            "us_only": self.config.get("us_only"),
-            "blacklist": self._config_to_list("qualification_blacklist"),
             "annotation": self.config.get("id"),
+            "qualifications": self._build_hit_qualifications(),
         }
         hit_info = self.mturkservice.create_hit(**hit_request)
-        if self.config.get("mode") == "sandbox":
-            lookup_url = (
-                "https://workersandbox.mturk.com/mturk/preview?groupId={type_id}"
-            )
-        else:
-            lookup_url = "https://worker.mturk.com/mturk/preview?groupId={type_id}"
+        url = hit_info["worker_url"]
 
         return {
-            "items": [lookup_url.format(**hit_info)],
+            "items": [url],
             "message": "HIT now published to Amazon Mechanical Turk",
         }
 
+    def compensate_worker(self, worker_id, email, dollars, notify=True):
+        """Pay a worker by means of a special HIT that only they can see.
+        """
+        qualification = self.mturkservice.create_qualification_type(
+            name="Dallinger Compensation Qualification - {}".format(
+                generate_random_id()
+            ),
+            description=(
+                "You have received a qualification to allow you to complete a "
+                "compensation HIT from Dallinger for ${}.".format(dollars)
+            ),
+        )
+        qid = qualification["id"]
+        self.mturkservice.assign_qualification(qid, worker_id, 1, notify=notify)
+        hit_request = {
+            "experiment_id": "(compensation only)",
+            "max_assignments": 1,
+            "title": "Dallinger Compensation HIT",
+            "description": "For compenation only; no task required.",
+            "keywords": [],
+            "reward": float(dollars),
+            "duration_hours": 1,
+            "lifetime_days": 3,
+            "question": MTurkQuestions.compensation(sandbox=self._is_sandbox),
+            "qualifications": [MTurkQualificationRequirements.must_have(qid)],
+            "do_subscribe": False,
+        }
+        hit_info = self.mturkservice.create_hit(**hit_request)
+        if email is not None:
+            message = {
+                "subject": "Dallinger Compensation HIT",
+                "sender": self.config.get("dallinger_email_address"),
+                "recipients": [email],
+                "body": (
+                    "A special compenstation HIT is available for you to complete on MTurk.\n\n"
+                    "Title: {title}\n"
+                    "Reward: ${reward:.2f}\n"
+                    "URL: {worker_url}"
+                ).format(**hit_info),
+            }
+
+            self.mailer.send(**message)
+        else:
+            message = {}
+
+        return {"hit": hit_info, "qualification": qualification, "email": message}
+
     def recruit(self, n=1):
         """Recruit n new participants to an existing HIT"""
         logger.info("Recruiting {} MTurk participants".format(n))
@@ -672,6 +721,24 @@ class MTurkRecruiter(Recruiter):
         # except MTurkServiceException as ex:
         #     logger.exception(str(ex))
 
+    @property
+    def _is_sandbox(self):
+        return self.config.get("mode") == "sandbox"
+
+    def _build_hit_qualifications(self):
+        quals = []
+        reqs = MTurkQualificationRequirements
+        if self.config.get("approve_requirement") is not None:
+            quals.append(reqs.min_approval(self.config.get("approve_requirement")))
+        if self.config.get("us_only"):
+            quals.append(reqs.restrict_to_countries(["US"]))
+        for item in self._config_to_list("qualification_blacklist"):
+            qtype = self.mturkservice.get_qualification_type_by_name(item)
+            if qtype:
+                quals.append(reqs.must_not_have(qtype["id"]))
+
+        return quals
+
     def _confirm_sns_subscription(self, token, topic):
         self.mturkservice.confirm_subscription(token=token, topic=topic)
 
@@ -738,7 +805,7 @@ class MTurkRecruiter(Recruiter):
 
     def _message_researcher(self, message):
         try:
-            self.messenger.send(message)
+            self.notifies_admin.send(message["subject"], message["body"])
         except MessengerError as ex:
             logger.exception(ex)
 
